#!/usr/bin/env python


def configuration(parent_package='', top_path=None):
    from numpy.distutils.misc_util import Configuration

    #lapack_opt = get_info('lapack_opt')
    config = Configuration('amg_core', parent_package, top_path)

    config.add_extension('_amg_core',
                         define_macros=[('__STDC_FORMAT_MACROS', 1)],
                         sources=['amg_core_wrap.cxx'])
                         #extra_info = lapack_opt)

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



from pyamg.testing import *

import numpy, scipy
from numpy import array, ones, arange, empty, array_split, zeros
from numpy.lib.arraysetops import setdiff1d
from scipy.sparse import csr_matrix, spdiags

from pyamg.gallery import poisson, load_example
from pyamg.strength import symmetric_strength_of_connection
from pyamg.aggregation.aggregate import standard_aggregation, naive_aggregation


class TestAggregate(TestCase):
    def setUp(self):
        self.cases = []

        # random matrices
        numpy.random.seed(0)
        for N in [2,3,5]:
            self.cases.append( csr_matrix(rand(N,N)) )

        # Poisson problems in 1D and 2D
        for N in [2,3,5,7,10,11,19]:
            self.cases.append( poisson( (N,), format='csr') )
        for N in [2,3,5,7,8]:
            self.cases.append( poisson( (N,N), format='csr') )

        for name in ['knot','airfoil','bar']:
            ex = load_example(name)
            self.cases.append( ex['A'].tocsr() )


    def test_standard_aggregation(self):
        for A in self.cases:
            S = symmetric_strength_of_connection(A)
            
            (expected,expected_Cpts) = reference_standard_aggregation(S)
            (result,Cpts)   = standard_aggregation(S)

            assert_equal( (result - expected).nnz, 0 )
            assert_equal( Cpts.shape[0], expected_Cpts.shape[0])
            assert_equal( setdiff1d(Cpts, expected_Cpts).shape[0], 0)
    
        # S is diagonal - no dofs aggregated
        S = spdiags([[1,1,1,1]],[0],4,4,format='csr')
        (result,Cpts)   = standard_aggregation(S)
        expected = array([[0],[0],[0],[0]])
        assert_equal(result.todense(),expected)
        assert_equal(Cpts.shape[0], 0)

    def test_naive_aggregation(self):
        for A in self.cases:
            S = symmetric_strength_of_connection(A)
            
            (expected,expected_Cpts) = reference_naive_aggregation(S)
            (result,Cpts)   = naive_aggregation(S)

            assert_equal( (result - expected).nnz, 0 )
            assert_equal( Cpts.shape[0], expected_Cpts.shape[0])
            assert_equal( setdiff1d(Cpts, expected_Cpts).shape[0], 0)
    
        # S is diagonal - no dofs aggregated
        S = spdiags([[1,1,1,1]],[0],4,4,format='csr')
        (result, Cpts)   = naive_aggregation(S)
        expected = numpy.eye(4)
        assert_equal(result.todense(),expected)
        assert_equal(Cpts.shape[0], 4)
        

class TestComplexAggregate(TestCase):
    def setUp(self):
        self.cases = []

        # Poisson problems in 2D
        for N in [2,3,5,7,8]:
            A = poisson( (N,N), format='csr'); A.data = A.data + 0.001j*rand(A.nnz)
            self.cases.append(A)

    def test_standard_aggregation(self):
        for A in self.cases:
            S = symmetric_strength_of_connection(A)
            
            (expected,expected_Cpts) = reference_standard_aggregation(S)
            (result,Cpts)   = standard_aggregation(S)

            assert_equal( (result - expected).nnz, 0 )
            assert_equal( Cpts.shape[0], expected_Cpts.shape[0])
            assert_equal( setdiff1d(Cpts, expected_Cpts).shape[0], 0)
    
    def test_naive_aggregation(self):
        for A in self.cases:
            S = symmetric_strength_of_connection(A)
            
            (expected,expected_Cpts) = reference_naive_aggregation(S)
            (result,Cpts)   = naive_aggregation(S)

            assert_equal( (result - expected).nnz, 0 )
            assert_equal( Cpts.shape[0], expected_Cpts.shape[0])
            assert_equal( setdiff1d(Cpts, expected_Cpts).shape[0], 0)


################################################
##   reference implementations for unittests  ##
################################################

# note that this method only tests the current implementation, not
# all possible implementations
def reference_standard_aggregation(C):
    S = array_split(C.indices,C.indptr[1:-1])

    n = C.shape[0]

    R = set(range(n))
    j = 0
    Cpts = []

    aggregates    = empty(n,dtype=C.indices.dtype)
    aggregates[:] = -1

    # Pass #1
    for i,row in enumerate(S):
        Ni = set(row) | set([i])

        if Ni.issubset(R):
            Cpts.append(i)
            R -= Ni
            for x in Ni:
                aggregates[x] = j
            j += 1

    # Pass #2
    Old_R = R.copy()
    for i,row in enumerate(S):
        if i not in R: continue

        for x in row:
            if x not in Old_R:
                aggregates[i] = aggregates[x]
                R.remove(i)
                break

    # Pass #3
    for i,row in enumerate(S):
        if i not in R: continue
        Ni = set(row) | set([i])
        Cpts.append(i)

        for x in Ni:
            if x in R:
                aggregates[x] = j
            j += 1

    assert(len(R) == 0)

    Pj = aggregates
    Pp = arange(n+1)
    Px = ones(n)

    return csr_matrix((Px,Pj,Pp)), array(Cpts)

#####################################

def reference_naive_aggregation(C):
    S = array_split(C.indices,C.indptr[1:-1])
    n = C.shape[0]
    aggregates    = empty(n, dtype=C.indices.dtype)
    aggregates[:] = -1 # aggregates[j] denotes the aggregate j is in
    R = zeros( (0,) )  # R stores already aggregated nodes
    j = 0              # j is the aggregate counter
    Cpts = []

    # Only one aggregation pass
    for i,row in enumerate(S):
        
        # if i isn't already aggregated, grab all his neighbors
        if aggregates[i] == -1:
            unaggregated_neighbors = numpy.setdiff1d(row, R)
            aggregates[unaggregated_neighbors] = j
            aggregates[i] = j
            j += 1
            R = numpy.union1d(R, unaggregated_neighbors)
            R = numpy.union1d(R, numpy.array([i]))
            Cpts.append(i)
        else:
            pass
    
    assert(numpy.unique(R).shape[0] == n)

    Pj = aggregates
    Pp = arange(n+1)
    Px = ones(n)

    return csr_matrix((Px,Pj,Pp)), array(Cpts)





"Matrix Gallery for Multigrid Solvers"

from info import __doc__

from elasticity import *
from example import *
from laplacian import *
from stencil import *
from mesh import *
from diffusion import *
from random_sparse import *
from demo import demo

__all__ = [s for s in dir() if not s.startswith('_')]
from pyamg.testing import Tester
test = Tester().test



from pyamg.testing import *

from numpy import matrix, array, diag, zeros, sqrt, abs, ravel, zeros_like, arange, dot
from scipy import rand, linalg, real, imag, mat, diag, random
from scipy.sparse import csr_matrix
from scipy.linalg import svd, eigvals

from pyamg.util.linalg import *
from pyamg import gallery

class TestLinalg(TestCase):
    def test_norm(self):
        cases = []

        cases.append(  4 )
        cases.append( -1 )
        cases.append( 2.5 )
        cases.append( 3 + 5j )
        cases.append( 7 - 2j )
        cases.append( [1 + 3j, 6] )
        cases.append( [1 + 3j, 6 - 2j] )

        for A in cases:
            assert_almost_equal( norm(A), linalg.norm(A) )

    def test_approximate_spectral_radius(self):
        cases = []

        cases.append( matrix([[-4]]) )
        cases.append( array([[-4]]) )
        
        cases.append( array([[2,0],[0,1]]) )
        cases.append( array([[-2,0],[0,1]]) )
      
        cases.append( array([[100,0,0],[0,101,0],[0,0,99]]) )
        
        for i in range(1,5):
            cases.append( rand(i,i) )
       
        # method should be almost exact for small matrices
        for A in cases:
            A = A.astype(float)
            Asp = csr_matrix(A)
            
            [E,V] = linalg.eig(A)
            E = abs(E)
            largest_eig = (E == E.max()).nonzero()[0]
            expected_eig = E[largest_eig]
            expected_vec = V[:,largest_eig]

            assert_almost_equal( approximate_spectral_radius(A),   expected_eig )
            assert_almost_equal( approximate_spectral_radius(Asp), expected_eig )
            vec = approximate_spectral_radius(A, return_vector=True)[1]
            diff = min( norm(expected_vec + vec), norm(expected_vec - vec) ) / norm(expected_vec)
            assert_almost_equal(diff, 0.0, decimal=4 )
            vec = approximate_spectral_radius(Asp, return_vector=True)[1]
            diff = min( norm(expected_vec + vec), norm(expected_vec - vec) ) / norm(expected_vec)
            assert_almost_equal(diff, 0.0, decimal=4 )
        
        # try symmetric matrices
        for A in cases:
            A = A + A.transpose()
            A = A.astype(float) 
            Asp = csr_matrix(A)

            [E,V] = linalg.eig(A)
            E = abs(E)
            largest_eig = (E == E.max()).nonzero()[0]
            expected_eig = E[largest_eig]
            expected_vec = V[:,largest_eig]

            assert_almost_equal( approximate_spectral_radius(A),   expected_eig )
            assert_almost_equal( approximate_spectral_radius(Asp), expected_eig )
            vec = approximate_spectral_radius(A, return_vector=True)[1]
            diff = min( norm(expected_vec + vec), norm(expected_vec - vec) ) / norm(expected_vec)
            assert_almost_equal(diff, 0.0, decimal=4 )
            vec = approximate_spectral_radius(Asp, return_vector=True)[1]
            diff = min( norm(expected_vec + vec), norm(expected_vec - vec) ) / norm(expected_vec)
            assert_almost_equal(diff, 0.0, decimal=4 )

      
        # test a larger matrix, and various parameter choices
        cases =[]
        A1 = gallery.poisson( (50,50), format='csr' )
        cases.append( (A1, 7.99241331495) )
        A2 = gallery.elasticity.linear_elasticity( (32,32), format='bsr' )[0]
        cases.append( (A2, 536549.922189) )
        for A,expected in cases:
            # test that increasing maxiter increases accuracy
            ans1 = approximate_spectral_radius(A, tol=1e-16, maxiter=5, restart=0)
            del A.rho
            ans2 = approximate_spectral_radius(A, tol=1e-16, maxiter=15, restart=0)
            del A.rho
            assert_equal( abs(ans2 - expected) < 0.5*abs(ans1 - expected), True )
            # test that increasing restart increases accuracy
            ans1 = approximate_spectral_radius(A, tol=1e-16, maxiter=10, restart=0)
            del A.rho
            ans2 = approximate_spectral_radius(A, tol=1e-16, maxiter=10, restart=1)
            del A.rho
            assert_equal( abs(ans2 - expected) < 0.8*abs(ans1 - expected), True  )
            # test tol 
            ans1 = approximate_spectral_radius(A, tol=0.1, maxiter=15, restart=5)
            del A.rho
            assert_equal( abs(ans1 - expected)/abs(expected) < 0.1, True  )
            ans2 = approximate_spectral_radius(A, tol=0.001, maxiter=15, restart=5)
            del A.rho
            assert_equal( abs(ans2 - expected)/abs(expected) < 0.001, True  )
            assert_equal( abs(ans2 - expected) < 0.1*abs(ans1 - expected), True  )

    def test_infinity_norm(self):
        A = matrix([[-4]])
        assert_equal(infinity_norm(csr_matrix(A)),4)

        A = matrix([[1,0,-5],[-2,5,0]])
        assert_equal(infinity_norm(csr_matrix(A)),7)

        A = matrix([[0,1],[0,-5]])
        assert_equal(infinity_norm(csr_matrix(A)),5)

        A = matrix([[1.3,-4.7,0],[-2.23,5.5,0],[9,0,-2]])
        assert_equal(infinity_norm(csr_matrix(A)),11)


class TestComplexLinalg(TestCase):    
    def test_approximate_spectral_radius(self):
        cases = []

        cases.append( matrix([[-4-4.0j]]) )
        cases.append( matrix([[-4+8.2j]]) )
        
        cases.append( matrix([[2.0-2.9j,0],[0,1.5]]) )
        cases.append( matrix([[-2.0-2.4j,0],[0,1.21]]) )
      
        cases.append( matrix([[100+1.0j,0,0],[0,101-1.0j,0],[0,0,99+9.9j]]) )
        
        for i in range(1,6):
            cases.append( matrix(rand(i,i)+1.0j*rand(i,i)) )
       
        # method should be almost exact for small matrices
        for A in cases:
            Asp = csr_matrix(A)
            [E,V] = linalg.eig(A)
            E = abs(E)
            largest_eig = (E == E.max()).nonzero()[0]
            expected_eig = E[largest_eig]
            expected_vec = V[:,largest_eig]

            assert_almost_equal( approximate_spectral_radius(A),   expected_eig )
            assert_almost_equal( approximate_spectral_radius(Asp), expected_eig )
            vec = approximate_spectral_radius(A, return_vector=True)[1]
            rayleigh = abs( dot(ravel(A*vec), ravel(vec)) / dot(ravel(vec), ravel(vec)) )
            assert_almost_equal(rayleigh, expected_eig, decimal=4 )
            vec = approximate_spectral_radius(Asp, return_vector=True)[1]
            rayleigh = abs( dot(ravel(Asp*vec), ravel(vec)) / dot(ravel(vec), ravel(vec)) )
            assert_almost_equal(rayleigh, expected_eig, decimal=4 )

            AA = mat(A).H*mat(A)
            AAsp = csr_matrix(AA)
            [E,V] = linalg.eig(AA)
            E = abs(E)
            largest_eig = (E == E.max()).nonzero()[0]
            expected_eig = E[largest_eig]
            expected_vec = V[:,largest_eig]

            assert_almost_equal( approximate_spectral_radius(AA),   expected_eig )
            assert_almost_equal( approximate_spectral_radius(AAsp), expected_eig )
            vec = approximate_spectral_radius(AA, return_vector=True)[1]
            rayleigh = abs( dot(ravel(AA*vec), ravel(vec)) / dot(ravel(vec), ravel(vec)) )
            assert_almost_equal(rayleigh, expected_eig, decimal=4 )
            vec = approximate_spectral_radius(AAsp, return_vector=True)[1]
            rayleigh = abs( dot(ravel(AAsp*vec), ravel(vec)) / dot(ravel(vec), ravel(vec)) )
            assert_almost_equal(rayleigh, expected_eig, decimal=4 )
 
    def test_infinity_norm(self):
        A = matrix([[-4-3.0j]])
        assert_equal(infinity_norm(csr_matrix(A)),5.0)

        A = matrix([[1,0,4.0-3.0j],[-2,5,0]])
        assert_equal(infinity_norm(csr_matrix(A)),7)

        A = matrix([[0,1],[0,-4.0+3.0j]])
        assert_equal(infinity_norm(csr_matrix(A)),5.0)


    def test_cond(self):
        # make tests repeatable
        random.seed(0)

        # Should be exact
        cases = []
        A = mat(array([2.14]))
        cases.append(A)
        A = mat(array([2.14j]))
        cases.append(A)
        A = mat(array([-1.2 + 2.14j]))
        cases.append(A)
        for i in range(1,6):
            A = mat(rand(i,i))
            cases.append(A)
            cases.append(1.0j*A)
            A = mat(A + 1.0j*rand(i,i))
            cases.append(A)

        for A in cases:
            U, Sigma, Vh = svd(A)
            exact = max(Sigma)/min(Sigma)
            c = cond(A)
            assert_almost_equal(exact, c)

    def test_condest(self):
        # make tests repeatable
        random.seed(0)
        
        # Should be exact for small matrices
        cases = []
        A = mat(array([2.14]))
        cases.append(A)
        A = mat(array([2.14j]))
        cases.append(A)
        A = mat(array([-1.2 + 2.14j]))
        cases.append(A)
        for i in range(1,6):
            A = mat(rand(i,i))
            cases.append(A)
            cases.append(1.0j*A)
            A = mat(A + 1.0j*rand(i,i))
            cases.append(A)

        for A in cases:
            eigs = eigvals(A)
            exact = max(abs(eigs))/min(abs(eigs))
            c = condest(A)
            assert_almost_equal(exact, c)

    def test_ishermitian(self):
        # make tests repeatable
        random.seed(0)
        casesT = []
        casesF = []
        # 1x1
        casesT.append( mat(rand(1,1)) )
        casesF.append( mat(1.0j*rand(1,1)) )
        # 2x2
        A = array([[1.0, 0.0], [2.0, 1.0]])
        Ai = 1.0j*A
        casesF.append(A)
        casesF.append(Ai)
        A = A + Ai
        casesF.append(A)
        casesT.append(A + A.conjugate().T)
        # 3x3
        A = mat(rand(3,3))
        Ai = 1.0j*rand(3,3)
        casesF.append(A)
        casesF.append(Ai)
        A = A + Ai
        casesF.append(A)
        casesT.append(A + A.H)

        for A in casesT:
            # dense arrays
            assert_equal(ishermitian(A, fast_check=False), True)
            assert_equal(ishermitian(A, fast_check=True), True)

            # csr arrays
            A = csr_matrix(A)
            assert_equal(ishermitian(A, fast_check=False), True)
            assert_equal(ishermitian(A, fast_check=True), True)

        for A in casesF:
            # dense arrays
            assert_equal(ishermitian(A, fast_check=False), False)
            assert_equal(ishermitian(A, fast_check=True), False)

            # csr arrays
            A = csr_matrix(A)
            assert_equal(ishermitian(A, fast_check=False), False)
            assert_equal(ishermitian(A, fast_check=True), False)


    def test_pinv_array(self):
        from scipy.linalg import pinv2
        
        tests = []
        tests.append(rand(1,1,1))
        tests.append(rand(3,1,1))
        tests.append(rand(1,2,2))
        tests.append(rand(3,2,2))
        tests.append(rand(1,3,3))
        tests.append(rand(3,3,3))
        A = rand(1,3,3)
        A[0,0,:] = A[0,1,:]
        tests.append(A)

        tests.append(rand(1,1,1) + 1.0j*rand(1,1,1))
        tests.append(rand(3,1,1) + 1.0j*rand(3,1,1))
        tests.append(rand(1,2,2) + 1.0j*rand(1,2,2))
        tests.append(rand(3,2,2) + 1.0j*rand(3,2,2))
        tests.append(rand(1,3,3) + 1.0j*rand(1,3,3))
        tests.append(rand(3,3,3) + 1.0j*rand(3,3,3))
        A = rand(1,3,3) + 1.0j*rand(1,3,3)
        A[0,0,:] = A[0,1,:]
        tests.append(A)

        for test in tests:
            pinv_test = zeros_like(test)
            for i in range(pinv_test.shape[0]):
                pinv_test[i] = pinv2(test[i])
            
            pinv_array(test)
            assert_array_almost_equal(test, pinv_test, decimal=4)




"""Compute coefficients for polynomial smoothers
"""

import numpy

__docformat__ = "restructuredtext en"

__all__ = ['chebyshev_polynomial_coefficients']

def chebyshev_polynomial_coefficients(a, b, degree):
    """Chebyshev polynomial coefficients for the interval [a,b]

    Parameters
    ----------
    a,b : float
        The left and right endpoints of the interval.
    degree : int
        Degree of desired Chebyshev polynomial

    Returns
    -------
    Coefficients of the Chebyshev polynomial C(t) with minimum
    magnitude on the interval [a,b] such that C(0) = 1.0.
    The coefficients are returned in descending order.
    
    Notes
    -----
    a,b typically represent the interval of the spectrum for some matrix
    that you wish to damp with a Chebyshev smoother.

    Examples
    --------
    >>> from pyamg.relaxation.chebyshev import chebyshev_polynomial_coefficients
    >>> print chebyshev_polynomial_coefficients(1.0,2.0, 3)
    [-0.32323232  1.45454545 -2.12121212  1.        ]
    """

    if a >= b or a <= 0:
        raise ValueError('invalid interval [%s,%s]' % (a,b))

    # Chebyshev roots for the interval [-1,1]
    std_roots = numpy.cos( numpy.pi * (numpy.arange(degree) + 0.5)/ degree )

    # Chebyshev roots for the interval [a,b]
    scaled_roots = 0.5 * (b-a) * (1 + std_roots) + a
    
    # Compute monic polynomial coefficients of polynomial with scaled roots
    scaled_poly  = numpy.poly(scaled_roots)

    # Scale coefficients to enforce C(0) = 1.0
    scaled_poly /= numpy.polyval(scaled_poly, 0)

    return scaled_poly



def mls_polynomial_coefficients(rho, degree):
    """Determine the coefficients for a MLS polynomial smoother
    
    Parameters
    ----------
    rho : {float}
        Spectral radius of the matrix in question
    degree : {int}
        Degree of polynomial coefficients to generate

    Returns
    -------
    Tuple of arrays (coeffs,roots) containing the
    coefficients for the (symmetric) polynomial smoother and
    the roots of polynomial prolongation smoother.

    The coefficients of the polynomial are in descending order

    References
    ----------
    .. [1] Parallel multigrid smoothing: polynomial versus Gauss--Seidel
       M. F. Adams, M. Brezina, J. J. Hu, and R. S. Tuminaro
       J. Comp. Phys., 188 (2003), pp. 593--610

    Examples
    --------
    >>> from pyamg.relaxation.chebyshev import mls_polynomial_coefficients
    >>> mls = mls_polynomial_coefficients(2.0, 2)
    >>> print mls[0] # coefficients
    [   6.4  -48.   144.  -220.   180.   -75.8   14.5]
    >>> print mls[1] # roots
    [ 1.4472136  0.5527864]
    """
    
    std_roots = numpy.cos( numpy.pi * (numpy.arange(degree) + 0.5)/ degree )
    #print std_roots

    roots = rho/2.0 * (1.0 - numpy.cos(2*numpy.pi*(numpy.arange(degree,dtype='float64') + 1)/(2.0*degree+1.0)))
    #print roots
    roots = 1.0/roots

    #S_coeffs = list(-numpy.poly(roots)[1:][::-1])

    S = numpy.poly(roots)[::-1]             #monomial coefficients of S error propagator
    
    SSA_max = rho/((2.0*degree+1.0)**2)    #upper bound on the spectral radius of S^2A
    S_hat = numpy.polymul(S,S) #monomial coefficients of \hat{S} propagator
    S_hat = numpy.hstack(( (-1.0/SSA_max)*S_hat, [1]) )

    coeffs = numpy.polymul(S_hat,S)          #coefficients for combined error propagator i.e. \hat{S}S
    coeffs = -coeffs[:-1]                    #coefficients for smoother

    return (coeffs,roots)


if __name__ == '__main__':
    if True:
        # show Chebyshev polynomial
        a = 1.0/100.0
        b = 1.0
        degree = 4
        
        coeffs = chebyshev_polynomial_coefficients(a, b, degree)
        print "coeffs",coeffs
        
        from pylab import *
        x = linspace(-0.1,1.1,100)
        plot(x, polyval(coeffs, x))
        vlines([a,b],-1,1)
        y = polyval(coeffs, a) 
        hlines([-y,y],a,b)
        ylim(-1.1,1.1)
        show()
    
    
    if False:
        # show MLS polynomial, currently broken?
        degree = 2
        rho    = 1.0
    
        coeffs,roots = mls_polynomial_coefficients(rho, degree)
        
        from pylab import *
        x = linspace(0.0,1.1,100)
        plot(x, polyval(coeffs, x))
        xlim(0, rho)
        show()




"Visualization Support"

from info import __doc__

from vtk_writer import *
from vis_coarse import *

__all__ = [s for s in dir() if not s.startswith('_')]
from pyamg.testing import Tester
test = Tester().test



from pyamg.testing import *

from numpy import matrix, array, diag, arange, ones, zeros_like, random
from scipy import rand
from scipy.linalg import norm
from pyamg.gallery import poisson, load_example 
from pyamg.blackbox import solver, solve, solver_configuration

class TestBlackbox(TestCase):
    def setUp(self):
        self.cases = []

        # Poisson problems in 1D and 2D
        N = 20
        self.cases.append( (poisson( (2*N,), format='csr'), rand(2*N,)) )     # 0
        self.cases.append( (poisson( (N,N), format='csr'), rand(N*N,)) )  # 1
        # Boxed examples
        A = load_example('recirc_flow')['A'].tocsr()                      # 2
        self.cases.append( (A, rand(A.shape[0],) ))
        A = load_example('bar')['A'].tobsr(blocksize=(3,3))               # 3
        self.cases.append( (A, rand(A.shape[0],) ))
                        

    def test_blackbox(self):
        for A,b in self.cases:
            x = solve(A,b,verb=False,maxiter=A.shape[0])
            assert( norm(b - A*x)/norm(b - A*rand(b.shape[0],)) < 1e-4 )

        ##
        # Special tests
        #(1) Make sure BSR format is preserved, and B is multiple vecs
        A,b = self.cases[-1]
        (x,ml) = solve(A,b,return_solver=True,verb=False)
        assert(ml.levels[0].B.shape[1] == 3)
        assert(ml.levels[0].A.format == 'bsr')
        
        #(2) Run with solver and make sure that solution is still good
        x = solve(A,b,existing_solver=ml,verb=False)
        assert( norm(b - A*x)/norm(b - A*rand(b.shape[0],)) < 1e-4 )

        #(3) Convert to CSR, make sure B is a single vector
        (x,ml) = solve(A.tocsr(),b,return_solver=True,verb=False)
        assert(ml.levels[0].B.shape[1] == 1)
        assert(ml.levels[0].A.format == 'csr')
         
        #(4) Run with x0, maxiter and tol
        x = solve(A,b,existing_solver=ml,x0=zeros_like(b), tol=1e-8, maxiter=300,verb=False)
        assert( norm(b - A*x)/norm(b - A*rand(b.shape[0],)) < 1e-7 )
 
        #(5) Run nonsymmetric example, make sure BH isn't None
        A,b = self.cases[2]
        (x,ml) = solve(A,b,return_solver=True,verb=False,maxiter=A.shape[0])
        assert(ml.levels[0].BH is not None)





"""Aggregation-based AMG"""

from adaptive import *
from aggregate import *
from aggregation import *
from tentative import *
from smooth import *
from rootnode import *

__all__ = [s for s in dir() if not s.startswith('_')]
from pyamg.testing import Tester
test = Tester().test



"""Compatible Relaxation"""

__docformat__ = "restructuredtext en"

import numpy as np
import scipy
from scipy.linalg import norm
from scipy.sparse import isspmatrix, csr_matrix, spdiags, isspmatrix_csr

from pyamg.relaxation import gauss_seidel, gauss_seidel_indexed

__all__ = ['CR', 'binormalize']


def CR(S, method='habituated', maxiter=20):
    """Use Compatible Relaxation to compute a C/F splitting

    Parameters
    ----------
    S : csr_matrix
        sparse matrix (n x n) usually matrix A of Ax=b
    method : {'habituated','concurrent'}
        Method used during relaxation:
            - concurrent: GS relaxation on F-points, leaving e_c = 0
            - habituated: full relaxation, setting e_c = 0
    maxiter : int
        maximum number of outer iterations (lambda)

    Returns
    -------
    splitting : array
        C/F list of 1's (coarse pt) and 0's (fine pt) (n x 1)

    References
    ----------
    .. [1] Livne, O.E., "Coarsening by compatible relaxation."
       Numer. Linear Algebra Appl. 11, No. 2-3, 205-227 (2004).

    Examples
    --------
    >>> from pyamg.gallery import poisson
    >>> from pyamg.classical.cr import CR
    >>> A = poisson((20,20),format='csr')
    >>> splitting = CR(A)

    """
    # parameters (paper notation)
    ntests = 3      # (nu) number of random tests to do per iteration
    nrelax = 4      # (eta) number of relaxation sweeps per test

    smagic = 1.0    # (s) parameter in [1,5] to account for fill-in
    gamma = 1.5     # (gamma) cycle index.  use 1.5 for 2d
    G = 30          # (G) number of equivalence classes (# of bins)
    tdepth = 1      # (t) drop depth on parse of L bins
    delta = 0       # (delta) drop threshold on parse of L bins
    alphai = 0.25   # (alpha_inc) quota increase

    # initializations
    alpha = 0.0     # coarsening ratio, quota
    beta = np.inf      # quality criterion
    beta1 = np.inf     # quality criterion, older
    beta2 = np.inf     # quality criterion, oldest
    n = S.shape[0]    # problem size
    nC = 0          # number of current Coarse points
    rhs = np.zeros((n, 1))  # rhs for Ae=0

    if not isspmatrix(S):
        raise TypeError('expecting sparse matrix')

    S = binormalize(S)

    splitting = np.zeros((S.shape[0], 1), dtype='intc')

    # out iterations ---------------
    for m in range(0, maxiter):

        mu = 0.0  # convergence rate
        E = np.zeros((n, 1))  # slowness measure

        # random iterations ---------------
        for k in range(0, ntests):

            e = 0.5*(1 + scipy.rand(n, 1))
            e[splitting > 0] = 0

            enorm = norm(e)

            # relaxation iterations ---------------
            for l in range(0, nrelax):

                if method == 'habituated':
                    gauss_seidel(S, e, np.zeros((n, 1)), iterations=1)
                    e[splitting > 0] = 0
                elif method == 'concurrent':
                    raise NotImplementedError('not implemented: need an \
                                                F-smoother')
                else:
                    raise NotImplementedError('method not recognized: need \
                                               habituated or concurrent')

                enorm_old = enorm
                enorm = norm(e)

                if enorm <= 1e-14:
                    # break out of loops
                    ntests = k
                    nrelax = l
                    maxiter = m
            # end relax

            # check slowness
            E = np.where(np.abs(e) > E, np.abs(e), E)

            # update convergence rate
            mu = mu + enorm/enorm_old
        # end random tests
        mu = mu/ntests

        # work
        alpha = float(nC)/n

        W = (1 + (smagic-1)*gamma*alpha)/(1-gamma*alpha)

        # quality criterion
        beta2 = beta1
        beta1 = beta
        beta = np.power(max([mu, 0.1]), 1.0 / W)

        # check if we're doing well
        if (beta > beta1 and beta1 > beta2) or \
           m == (maxiter-1) or max(E) < 1e-13:
            return splitting.ravel()

        # now add points
        #
        # update limit on additions to splitting (C)
        if alpha < 1e-13:
            alpha = 0.25
        else:
            alpha = (1-alphai) * alpha + alphai * (1/gamma)

        nCmax = np.ceil(alpha * n)

        L = np.ceil(G * E / E.max()).ravel()

        binid = G

        # add whole bins (and t-depth nodes) at a time
        u = np.zeros((n, 1))
        # TODO This loop may never halt...
        #      Perhaps loop over nC < nCmax and binid > 0 ?
        while nC < nCmax:
            if delta > 0:
                raise NotImplementedError
            if tdepth != 1:
                raise NotImplementedError

            (roots,) = np.where(L == binid)

            for root in roots:
                if L[root] >= 0:
                    cols = S[root, :].indices
                    splitting[root] = 1    # add roots
                    nC += 1
                    L[cols] = -1
            binid -= 1

            #L[troots] = -1          # mark t-rings visited
            #u[:]=0.0
            #u[roots] = 1.0
            #for depth in range(0,tdepth):
            #    u = np.abs(S) * u
            #(troots,tmp) = np.where(u>0)

    return splitting.ravel()


def _CRsweep(A, Findex, Cindex, nu, thetacr, method):

    n = A.shape[0]    # problem size
    numax = nu
    z = np.zeros((n,))
    e = np.ones((n,))
    e[Cindex] = 0.0
    enorm = norm(e)
    rhok = 1

    for it in range(1, numax+1):

        if method == 'habituated':
            gauss_seidel(A, e, z, iterations=1)
            e[Cindex] = 0.0
        elif method == 'concurrent':
            gauss_seidel_indexed(A, e, z, indices=Findex, iterations=1)
        else:
            raise NotImplementedError('method not recognized: need \
                                       habituated or concurrent')

        enorm_old = enorm
        enorm = norm(e)
        rhok_old = rhok
        rhok = enorm / enorm_old

        # criteria 1
        if (abs(rhok - rhok_old) / rhok < 0.1) and (it >= nu):
            return rhok, e

        # criteria 2
        if rhok < 0.1 * thetacr:
            return rhok, e


def CRalpha(A, method='habituated', nu=3, thetacr=0.7, thetacs=[0.3, 0.5],
            maxiter=20):
    """
    >>> from pyamg.gallery import poisson
    >>> from cr import CRalpha
    >>> A = poisson((20,20),format='csr')
    >>> splitting = CRalpha(A)
    """
    n = A.shape[0]    # problem size

    thetacs = list(thetacs)
    thetacs.reverse()

    if not isspmatrix_csr(A):
        raise TypeError('expecting csr sparse matrix A')

    if A.dtype == complex:
        raise NotImplementedError('complex A not implemented')

    # 3.1a
    splitting = np.zeros((n,), dtype='intc')
    gamma = np.zeros((n,))

    # 3.1b
    Cindex = np.where(splitting == 1)[0]
    Findex = np.where(splitting == 0)[0]
    rho, e = _CRsweep(A, Findex, Cindex, nu, thetacr, method=method)

    # 3.1c
    for it in range(0, maxiter):

        print it
        # 3.1d (assuming constant initial e in _CRsweep)
        # should already be zero at C pts (Cindex)
        gamma[Findex] = np.abs(e[Findex]) / np.abs(e[Findex]).max()

        # 3.1e
        Uindex = np.where(gamma > thetacs[0])[0]
        if len(thetacs) > 1:
            thetacs.pop()

        # 3.1f
        # first find the weights: omega_i = |N_i\C| + gamma_i
        omega = -np.inf * np.ones((n,))
        for i in Uindex:
            J = A.indices[np.arange(A.indptr[i], A.indptr[i+1])]
            J = np.where(splitting[J] == 0)[0]
            omega[i] = len(J) + gamma[i]

        # independent set
        Usize = len(Uindex)
        while Usize > 0:
            # step 1
            i = omega.argmax()
            splitting[i] = 1
            gamma[i] = 0.0
            # step 2
            J = A.indices[np.arange(A.indptr[i], A.indptr[i+1])]
            J = np.intersect1d(J, Uindex, assume_unique=True)
            omega[i] = -np.inf
            omega[J] = -np.inf

            # step 3
            for j in J:
                K = A.indices[np.arange(A.indptr[j], A.indptr[j+1])]
                K = np.intersect1d(K, Uindex, assume_unique=True)
                omega[K] = omega[K] + 1.0

            Usize -= 1

        Cindex = np.where(splitting == 1)[0]
        Findex = np.where(splitting == 0)[0]
        rho, e = _CRsweep(A, Findex, Cindex, nu, thetacr, method=method)

        print rho
        if rho < thetacr:
            break

    return splitting


def binormalize(A, tol=1e-5, maxiter=10):
    """Binormalize matrix A.  Attempt to create unit l_1 norm rows.

    Parameters
    ----------
    A : csr_matrix
        sparse matrix (n x n)
    tol : float
        tolerance
    x : array
        guess at the diagonal
    maxiter : int
        maximum number of iterations to try

    Returns
    -------
    C : csr_matrix
        diagonally scaled A, C=DAD

    Notes
    -----
        - Goal: Scale A so that l_1 norm of the rows are equal to 1:
        - B = DAD
        - want row sum of B = 1
        - easily done with tol=0 if B=DA, but this is not symmetric
        - algorithm is O(N log (1.0/tol))

    Examples
    --------
    >>> from pyamg.gallery import poisson
    >>> from pyamg.classical import binormalize
    >>> A = poisson((10,),format='csr')
    >>> C = binormalize(A)

    References
    ----------
    .. [1] Livne, Golub, "Scaling by Binormalization"
       Tech Report SCCM-03-12, SCCM, Stanford, 2003
       http://citeseerx.ist.psu.edu/viewdoc/summary?doi=10.1.1.3.1679

    """
    if not isspmatrix(A):
        raise TypeError('expecting sparse matrix A')

    if A.dtype == complex:
        raise NotImplementedError('complex A not implemented')

    n = A.shape[0]
    it = 0
    x = np.ones((n, 1)).ravel()

    # 1.
    B = A.multiply(A).tocsc()  # power(A,2) inconsistent in numpy, scipy.sparse
    d = B.diagonal().ravel()

    # 2.
    beta = B * x
    betabar = (1.0/n) * np.dot(x, beta)
    stdev = rowsum_stdev(x, beta)

    #3
    while stdev > tol and it < maxiter:
        for i in range(0, n):
            # solve equation x_i, keeping x_j's fixed
            # see equation (12)
            c2 = (n-1)*d[i]
            c1 = (n-2)*(beta[i] - d[i]*x[i])
            c0 = -d[i]*x[i]*x[i] + 2*beta[i]*x[i] - n*betabar
            if (-c0 < 1e-14):
                print 'warning: A nearly un-binormalizable...'
                return A
            else:
                # see equation (12)
                xnew = (2*c0)/(-c1 - np.sqrt(c1*c1 - 4*c0*c2))
            dx = xnew - x[i]

            # here we assume input matrix is symmetric since we grab a row of B
            # instead of a column
            ii = B.indptr[i]
            iii = B.indptr[i+1]
            dot_Bcol = np.dot(x[B.indices[ii:iii]], B.data[ii:iii])

            betabar = betabar + (1.0/n)*dx*(dot_Bcol + beta[i] + d[i]*dx)
            beta[B.indices[ii:iii]] += dx*B.data[ii:iii]

            x[i] = xnew

        stdev = rowsum_stdev(x, beta)
        it += 1

    # rescale for unit 2-norm
    d = np.sqrt(x)
    D = spdiags(d.ravel(), [0], n, n)
    C = D * A * D
    C = C.tocsr()
    beta = C.multiply(C).sum(axis=1)
    scale = np.sqrt((1.0/n) * np.sum(beta))
    return (1/scale)*C


def rowsum_stdev(x, beta):
    """Compute row sum standard deviation

    Compute for approximation x, the std dev of the row sums
    s(x) = ( 1/n \sum_k  (x_k beta_k - betabar)^2 )^(1/2)
    with betabar = 1/n dot(beta,x)

    Parameters
    ----------
    x : array
    beta : array

    Returns
    -------
    s(x)/betabar : float

    Notes
    -----
    equation (7) in Livne/Golub

    """
    n = x.size
    betabar = (1.0/n) * np.dot(x, beta)
    stdev = np.sqrt((1.0/n) *
                    np.sum(np.power(np.multiply(x, beta) - betabar, 2)))
    return stdev/betabar



from pyamg.testing import *

import numpy
from numpy import array, zeros, mat, eye, ones, setdiff1d, min, ravel, diff, mod, repeat, sqrt, finfo, abs
from scipy import rand, real, imag, mat, zeros, sign, eye, arange
from scipy.sparse import csr_matrix, isspmatrix_csr, bsr_matrix, isspmatrix_bsr, spdiags, coo_matrix
import scipy.sparse
from scipy.special import round
from scipy.linalg import pinv2, pinv

from pyamg.gallery import poisson, linear_elasticity, load_example, stencil_grid
from pyamg.strength import *
from pyamg.amg_core import incomplete_mat_mult_csr
from pyamg.util.linalg import approximate_spectral_radius
from pyamg.util.utils import scale_rows


class TestStrengthOfConnection(TestCase):
    def setUp(self):
        self.cases = []

        # random matrices
        numpy.random.seed(0)
        for N in [2,3,5]:
            self.cases.append( csr_matrix(rand(N,N)) )

        # Poisson problems in 1D and 2D
        for N in [2,3,5,7,10,11,19]:
            self.cases.append( poisson( (N,), format='csr') )
        for N in [2,3,7,9]:
            self.cases.append( poisson( (N,N), format='csr') )

        for name in ['knot','airfoil','bar']:
            ex = load_example(name)
            self.cases.append( ex['A'].tocsr() )

    def test_classical_strength_of_connection(self):
        for A in self.cases:
            for theta in [ 0.0, 0.05, 0.25, 0.50, 0.90 ]:
                result   = classical_strength_of_connection(A, theta)
                expected = reference_classical_strength_of_connection(A, theta)
                
                assert_equal( result.nnz, expected.nnz )
                assert_array_almost_equal( result.todense(), expected.todense() )

    def test_symmetric_strength_of_connection(self):
        for A in self.cases:
            for theta in [0.0, 0.1, 0.5, 1.0, 10.0]:
                result   = symmetric_strength_of_connection(A, theta)
                expected = reference_symmetric_strength_of_connection(A, theta)

                assert_equal( result.nnz,       expected.nnz)
                assert_array_almost_equal( result.todense(), expected.todense() )

    def test_distance_strength_of_connection(self):
        data = load_example('airfoil')
        cases = []
        cases.append( (data['A'].tocsr(), data['vertices']) )

        for (A,V) in cases:
            for theta in [1.5, 2.0, 2.5]:
                result = distance_strength_of_connection(A, V, theta=theta)
                expected = reference_distance_strength_of_connection(A, V, theta=theta)
                assert_equal( result.nnz,       expected.nnz)
                assert_array_almost_equal( result.todense(), expected.todense() )

        for (A,V) in cases:
            for theta in [0.5, 1.0, 1.5]:
                result = distance_strength_of_connection(A, V, theta=theta, \
                                            relative_drop=False)
                expected = reference_distance_strength_of_connection(A, V, \
                                            theta=theta, relative_drop=False)
                assert_equal( result.nnz,       expected.nnz)
                assert_array_almost_equal( result.todense(), expected.todense() )



    def test_incomplete_mat_mult_csr(self):
        # Test a critical helper routine for evolution_strength_of_connection(...)
        # We test that (A*B).multiply(mask) = incomplete_mat_mult_csr(A,B,mask)
        #
        # This function assumes square matrices, A, B and mask, so that simplifies the testing
        cases = []

        # 1x1 tests
        A = csr_matrix(mat([[1.1]]))
        B = csr_matrix(mat([[1.0]]))
        A2 = csr_matrix(mat([[0.]]))
        mask = csr_matrix(mat([[1.]]))
        cases.append( (A,A,mask) ) 
        cases.append( (A,B,mask) ) 
        cases.append( (A,A2,mask) ) 
        cases.append( (A2,A2,mask) ) 

        # 2x2 tests
        A = csr_matrix(mat([[1.,2.],[2.,4.]]))
        B = csr_matrix(mat([[1.3,2.],[2.8,4.]]))
        A2 = csr_matrix(mat([[1.3,0.],[0.,4.]]))
        B2 = csr_matrix(mat([[1.3,0.],[2.,4.]]))
        mask = csr_matrix( (ones(4),(array([0,0,1,1]),array([0,1,0,1]))), shape=(2,2) )
        cases.append( (A,A,mask) ) 
        cases.append( (A,B,mask) ) 
        cases.append( (A2,A2,mask) ) 
        cases.append( (A2,B2,mask) ) 

        mask = csr_matrix( (ones(3),(array([0,0,1]),array([0,1,1]))), shape=(2,2) )
        cases.append( (A,A,mask) ) 
        cases.append( (A,B,mask) ) 
        cases.append( (A2,A2,mask) ) 
        cases.append( (A2,B2,mask) ) 

        mask = csr_matrix( (ones(2),(array([0,1]),array([0,0]))), shape=(2,2) )
        cases.append( (A,A,mask) ) 
        cases.append( (A,B,mask) ) 
        cases.append( (A2,A2,mask) ) 
        cases.append( (A2,B2,mask) ) 

        # 5x5 tests
        A = mat([[  0. ,  16.9,   6.4,   0.0,   5.8],
                    [ 16.9,  13.8,   7.2,   0. ,   9.5],
                    [  6.4,   7.2,  12. ,   6.1,   5.9],
                    [  0.0,   0. ,   6.1,   0. ,   0. ],
                    [  5.8,   9.5,   5.9,   0. ,  13. ]])
        C = A.copy()
        C[1,0] = 3.1
        C[3,2] = 10.1
        A2 = A.copy()
        A2[1,:] = 0.0
        A3 = A2.copy()
        A3[:,1] = 0.0
        A = csr_matrix(A)
        A2 = csr_matrix(A2)
        A3 = csr_matrix(A3)
        C = csr_matrix(C)

        mask = A.copy()
        mask.data[:] = 1.0
        cases.append( (A,A,mask) )
        cases.append( (C,C,mask) )
        cases.append( (A2,A2,mask) )
        cases.append( (A3,A3,mask) )
        cases.append( (A,A2,mask) )
        cases.append( (A3,A,mask) )
        cases.append( (A,C,mask) )
        cases.append( (C,A,mask) )

        mask.data[1] = 0.0        
        mask.data[5] = 0.0        
        mask.data[9] = 0.0        
        mask.data[13] = 0.0        
        mask.eliminate_zeros()
        cases.append( (A,A,mask) )
        cases.append( (C,C,mask) )
        cases.append( (A2,A2,mask) )
        cases.append( (A3,A3,mask) )
        cases.append( (A,A2,mask) )
        cases.append( (A3,A,mask) )
        cases.append( (A,C,mask) )
        cases.append( (C,A,mask) )

        # Laplacian tests
        A = poisson((5,5),format='csr')
        B = A.copy()
        B.data[1] = 3.5
        B.data[11] = 11.6
        B.data[28] = -3.2
        C = csr_matrix(zeros(A.shape))
        mask=A.copy()
        mask.data[:]=1.0
        cases.append( (A,A,mask) )
        cases.append( (A,B,mask) )
        cases.append( (B,A,mask) )
        cases.append( (C,A,mask) )
        cases.append( (A,C,mask) )
        cases.append( (C,C,mask) )

        # Imaginary tests
        A = mat([[  0.0 +0.j ,   0.0+16.9j,   6.4 +1.2j,   0.0 +0.j ,   0.0 +0.j ],
                 [ 16.9 +0.j ,  13.8 +0.j ,   7.2 +0.j ,   0.0 +0.j ,   0.0 +9.5j],
                 [  0.0 +6.4j,   7.2 -8.1j,  12.0 +0.j ,   6.1 +0.j ,   5.9 +0.j ],
                 [  0.0 +0.j ,   0.0 +0.j ,   6.1 +0.j ,   0.0 +0.j ,   0.0 +0.j ],
                 [  5.8 +0.j ,  -4.0 +9.5j,  -3.2 -5.9j,   0.0 +0.j ,  13.0 +0.j ]])
        C = A.copy()
        C[1,0] = 3.1j - 1.3
        C[3,2] = -10.1j + 9.7
        A = csr_matrix(A)
        C = csr_matrix(C)

        mask = A.copy()
        mask.data[:] = 1.0
        cases.append( (A,A,mask) )
        cases.append( (C,C,mask) )
        cases.append( (A,C,mask) )
        cases.append( (C,A,mask) )


        for case in cases:
            A = case[0].tocsr()
            B = case[1].tocsc()
            mask = case[2].tocsr()
            A.sort_indices()
            B.sort_indices()
            mask.sort_indices()
            result = mask.copy()
            incomplete_mat_mult_csr(A.indptr,A.indices,A.data, B.indptr,B.indices,B.data, 
                              result.indptr,result.indices,result.data, A.shape[0])
            result.eliminate_zeros()
            exact = (A*B).multiply(mask)
            exact.sort_indices()
            exact.eliminate_zeros()
            assert_array_almost_equal(exact.data, result.data)
            assert_array_equal(exact.indptr, result.indptr)
            assert_array_equal(exact.indices, result.indices)


    def test_evolution_strength_of_connection(self):
        # Params:  A, B, epsilon=4.0, k=2, proj_type="l2"
        cases = []

        # Ensure that isotropic diffusion results in isotropic strength stencil
        for N in [3,5,7,10]:
            A = poisson( (N,), format='csr')
            B = ones((A.shape[0],1))
            cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})

        # Ensure that anisotropic diffusion results in an anisotropic strength stencil
        for N in [3,6,7]:
            A = spdiags([-ones(N*N), -0.001*ones(N*N), 2.002*ones(N*N),-0.001*ones(N*N),-ones(N*N)],[-N, -1, 0, 1, N], N*N, N*N, format='csr')
            B = ones((A.shape[0],1))
            cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
            
        # Ensure that isotropic elasticity results in an isotropic stencil
        for N in [3,6,7]:
            (A,B) = linear_elasticity( (N,N), format='bsr')
            cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 32.0, 'k' : 8, 'proj' : 'D_A'})

        # Run an example with a non-uniform stencil
        ex = load_example('airfoil')
        A = ex['A'].tocsr()
        B = ones((A.shape[0],1))
        cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 8.0, 'k' : 4, 'proj' : 'D_A'})
        Absr = A.tobsr(blocksize=(5,5))
        cases.append({'A' : Absr.copy(), 'B' : B.copy(), 'epsilon' : 8.0, 'k' : 4, 'proj' : 'D_A'})
        # Different B
        B = arange(1, 2*A.shape[0]+1, dtype=float).reshape(-1,2)
        cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
        cases.append({'A' : Absr.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
       
        # Zero row and column 
        A.data[A.indptr[4]:A.indptr[5]] = 0.0
        A = A.tocsc()
        A.data[A.indptr[4]:A.indptr[5]] = 0.0
        A.eliminate_zeros();  A = A.tocsr();  A.sort_indices()
        cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
        Absr = A.tobsr(blocksize=(5,5))
        cases.append({'A' : Absr.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
        
        for ca in cases:
            scipy.random.seed(0)  #make results deterministic
            result = evolution_strength_of_connection(ca['A'], ca['B'], epsilon=ca['epsilon'], \
                                 k=ca['k'], proj_type=ca['proj'], symmetrize_measure=False)
            scipy.random.seed(0)  #make results deterministic
            expected = reference_evolution_strength_of_connection(ca['A'], ca['B'], epsilon=ca['epsilon'], \
                                 k=ca['k'], proj_type=ca['proj'])
            assert_array_almost_equal( result.todense(), expected.todense(), decimal=4)

        # Test Scale Invariance for multiple near nullspace candidates
        (A,B) = linear_elasticity( (5,5), format='bsr')
        scipy.random.seed(0)  #make results deterministic
        result_unscaled = evolution_strength_of_connection(A, B, epsilon=4.0, \
                           k=2, proj_type="D_A", symmetrize_measure=False)
        # create scaled A
        D = spdiags([arange(A.shape[0],2*A.shape[0],dtype=float)], \
                         [0], A.shape[0], A.shape[0], format = 'csr')
        Dinv = spdiags([1.0/arange(A.shape[0],2*A.shape[0],dtype=float)], \
                                [0], A.shape[0], A.shape[0], format = 'csr')
        scipy.random.seed(0)  #make results deterministic
        result_scaled = evolution_strength_of_connection( (D*A*D).tobsr(blocksize=(2,2)), \
                  Dinv*B, epsilon=4.0, k=2, proj_type="D_A", symmetrize_measure=False)
        assert_array_almost_equal( result_scaled.todense(), result_unscaled.todense(), decimal=2 )


# Define Complex tests
class TestComplexStrengthOfConnection(TestCase):
    def setUp(self):
        self.cases = []

        # random matrices
        numpy.random.seed(0)
        for N in [2,3,5]:
            self.cases.append( csr_matrix(rand(N,N)) + csr_matrix(1.0j*rand(N,N)))

        # Poisson problems in 1D and 2D
        for N in [2,3,5,7,10,11,19]:
            A = poisson( (N,), format='csr'); A.data = A.data + 1.0j*A.data;
            self.cases.append(A)
        for N in [2,3,7,9]:
            A = poisson( (N,N), format='csr'); A.data = A.data + 1.0j*rand(A.data.shape[0],);
            self.cases.append(A)

        for name in ['knot','airfoil','bar']:
            ex = load_example(name)
            A = ex['A'].tocsr(); A.data = A.data + 0.5j*rand(A.data.shape[0],);
            self.cases.append(A)

    def test_classical_strength_of_connection(self):
        for A in self.cases:
            for theta in [ 0.0, 0.05, 0.25, 0.50, 0.90 ]:
                result   = classical_strength_of_connection(A, theta)
                expected = reference_classical_strength_of_connection(A, theta)
                
                assert_equal( result.nnz, expected.nnz )
                assert_array_almost_equal( result.todense(), expected.todense() )

    def test_symmetric_strength_of_connection(self):
        for A in self.cases:
            for theta in [0.0, 0.1, 0.5, 1.0, 10.0]:
                expected = reference_symmetric_strength_of_connection(A, theta)
                result   = symmetric_strength_of_connection(A, theta)

                assert_equal( result.nnz,       expected.nnz)
                assert_array_almost_equal( result.todense(), expected.todense() )

    def test_evolution_strength_of_connection(self):
        cases = [] 

        # Single near nullspace candidate
        stencil = [[0.0, -1.0, 0.0],[-0.001, 2.002, -0.001],[0.0, -1.0, 0.0]]
        A = 1.0j*stencil_grid(stencil, (4,4), format='csr')
        B = 1.0j*ones((A.shape[0],1))
        B[0] = 1.2 - 12.0j
        B[11] = -14.2
        cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})

        # Multiple near nullspace candidate
        B = 1.0j*ones((A.shape[0],2))
        B[0:-1:2,0] = 0.0
        B[1:-1:2,1] = 0.0
        B[-1,0] = 0.0;  B[11,1] = -14.2;  B[0,0] = 1.2 - 12.0j
        cases.append({'A' : A.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
        Absr = A.tobsr(blocksize=(2,2))
        cases.append({'A' : Absr.copy(), 'B' : B.copy(), 'epsilon' : 4.0, 'k' : 2, 'proj' : 'l2'})
        
        for ca in cases:
            scipy.random.seed(0)  #make results deterministic
            result = evolution_strength_of_connection(ca['A'], ca['B'], epsilon=ca['epsilon'], \
                                 k=ca['k'], proj_type=ca['proj'], symmetrize_measure=False)
            scipy.random.seed(0)  #make results deterministic
            expected = reference_evolution_strength_of_connection(ca['A'], ca['B'], epsilon=ca['epsilon'], \
                                                           k=ca['k'], proj_type=ca['proj'])
            assert_array_almost_equal( result.todense(), expected.todense() )

        # Test Scale Invariance for a single candidate
        A = 1.0j*poisson( (5,5), format='csr')
        B = 1.0j*arange(1,A.shape[0]+1,dtype=float).reshape(-1,1)
        scipy.random.seed(0)  #make results deterministic
        result_unscaled = evolution_strength_of_connection(A, B, epsilon=4.0, k=2, proj_type="D_A", symmetrize_measure=False)
        # create scaled A
        D = spdiags([arange(A.shape[0],2*A.shape[0],dtype=float)], [0], A.shape[0], A.shape[0], format = 'csr')
        Dinv = spdiags([1.0/arange(A.shape[0],2*A.shape[0],dtype=float)], [0], A.shape[0], A.shape[0], format = 'csr')
        scipy.random.seed(0)  #make results deterministic
        result_scaled = evolution_strength_of_connection(D*A*D, Dinv*B, epsilon=4.0, k=2, proj_type="D_A", symmetrize_measure=False)
        assert_array_almost_equal( result_scaled.todense(), result_unscaled.todense(), decimal=2 )
        
        # Test that the l2 and D_A are the same for the 1 candidate case
        scipy.random.seed(0)  #make results deterministic
        resultDA = evolution_strength_of_connection(D*A*D, Dinv*B, epsilon=4.0, \
                            k=2, proj_type="D_A", symmetrize_measure=False)
        scipy.random.seed(0)  #make results deterministic
        resultl2 = evolution_strength_of_connection(D*A*D, Dinv*B, epsilon=4.0, \
                              k=2, proj_type="l2", symmetrize_measure=False)
        assert_array_almost_equal( resultDA.todense(), resultl2.todense() )

        # Test Scale Invariance for multiple candidates
        (A,B) = linear_elasticity( (5,5), format='bsr')
        A = 1.0j*A
        B = 1.0j*B
        scipy.random.seed(0)  #make results deterministic
        result_unscaled = evolution_strength_of_connection(A, B, epsilon=4.0, k=2, \
                                     proj_type="D_A", symmetrize_measure=False)
        # create scaled A
        D = spdiags([arange(A.shape[0],2*A.shape[0],dtype=float)], [0], A.shape[0], A.shape[0], format = 'csr')
        Dinv = spdiags([1.0/arange(A.shape[0],2*A.shape[0],dtype=float)], [0], A.shape[0], A.shape[0], format = 'csr')
        scipy.random.seed(0)  #make results deterministic
        result_scaled = evolution_strength_of_connection((D*A*D).tobsr(blocksize=(2,2)), Dinv*B, \
                                epsilon=4.0, k=2, proj_type="D_A", symmetrize_measure=False)
        assert_array_almost_equal( result_scaled.todense(), result_unscaled.todense(), decimal=2 )


################################################
##   reference implementations for unittests  ##
################################################
def reference_classical_strength_of_connection(A, theta):
    # This complex extension of the classic Ruge-Stuben 
    # strength-of-connection has some theoretical justification in
    # "AMG Solvers for Complex-Valued Matrices", Scott MacClachlan, 
    # Cornelis Oosterlee

    # Connection is strong if, 
    #   | a_ij| >= theta * max_{k != i} |a_ik|
    S = coo_matrix(A)
    
    # remove diagonals
    mask = S.row != S.col
    S.row  = S.row[mask]
    S.col  = S.col[mask]
    S.data = S.data[mask]
    max_offdiag    = numpy.empty(S.shape[0])
    max_offdiag[:] = numpy.finfo(S.data.dtype).min

    # Note abs(.) takes the complex modulus
    for i,v in zip(S.row,S.data):
        max_offdiag[i] = max(max_offdiag[i], abs(v))

    # strong connections
    mask = abs(S.data) >= (theta * max_offdiag[S.row])
    S.row  = S.row[mask]
    S.col  = S.col[mask]
    S.data = S.data[mask]
    
    # Add back diagonal
    D = scipy.sparse.eye(S.shape[0], S.shape[0], format="csr", dtype=A.dtype)
    D.data[:] = csr_matrix(A).diagonal()
    S = S.tocsr() + D
    
    # Strength represents "distance", so take the magnitude 
    S.data = abs(S.data)

    # Scale S by the largest magnitude entry in each row
    largest_row_entry = numpy.zeros((S.shape[0],), dtype=S.dtype)
    for i in range(S.shape[0]):
        for j in range(S.indptr[i], S.indptr[i+1]):
            val = numpy.abs(S.data[j])
            if val > largest_row_entry[i]:
                largest_row_entry[i] = val
    
    largest_row_entry[ largest_row_entry != 0 ] = 1.0 / largest_row_entry[ largest_row_entry != 0 ] 
    S = S.tocsr()   
    S = scale_rows(S, largest_row_entry, copy=True)

    return S
    

def reference_symmetric_strength_of_connection(A, theta):
    # This is just a direct complex extension of the classic 
    # SA strength-of-connection measure.  The extension continues 
    # to compare magnitudes. This should reduce to the classic 
    # measure if A is all real.

    #if theta == 0:
    #    return A
    
    D = abs(A.diagonal())

    S = coo_matrix(A)

    mask  = S.row != S.col
    DD = array(D[S.row] * D[S.col]).reshape(-1,)
    # Note that abs takes the complex modulus element-wise
    # Note that using the square of the measure is the technique used 
    # in the C++ routine, so we use it here.  Doing otherwise causes errors. 
    mask &= (  (real(S.data)**2 + imag(S.data)**2) >= theta*theta*DD )

    S.row  = S.row[mask]
    S.col  = S.col[mask]
    S.data = S.data[mask]

    # Add back diagonal
    D = scipy.sparse.eye(S.shape[0], S.shape[0], format="csr", dtype=A.dtype)
    D.data[:] = csr_matrix(A).diagonal()
    S = S.tocsr() + D

    # Strength represents "distance", so take the magnitude 
    S.data = abs(S.data)

    # Scale S by the largest magnitude entry in each row
    largest_row_entry = numpy.zeros((S.shape[0],), dtype=S.dtype)
    for i in range(S.shape[0]):
        for j in range(S.indptr[i], S.indptr[i+1]):
            val = numpy.abs(S.data[j])
            if val > largest_row_entry[i]:
                largest_row_entry[i] = val
    
    largest_row_entry[ largest_row_entry != 0 ] = 1.0 / largest_row_entry[ largest_row_entry != 0 ] 
    S = S.tocsr()   
    S = scale_rows(S, largest_row_entry, copy=True)
     
    return S



def reference_evolution_strength_of_connection(A, B, epsilon=4.0, k=2, proj_type="l2"):
    """
    All python reference implementation for Evolution Strength of Connection
    
    --> If doing imaginary test, both A and B should be imaginary type upon entry
    --> This does the "unsymmetrized" version of the ode measure
    """
   
    #number of PDEs per point is defined implicitly by block size
    csrflag = isspmatrix_csr(A)
    if csrflag:
        numPDEs = 1
    else:
        numPDEs = A.blocksize[0]
        A = A.tocsr()
    
    # Preliminaries
    near_zero = finfo(float).eps
    sqrt_near_zero = sqrt(sqrt(near_zero))
    Bmat = mat(B)
    A.eliminate_zeros()
    A.sort_indices()
    dimen = A.shape[1]
    NullDim = Bmat.shape[1]

    #Get spectral radius of Dinv*A, this is the time step size for the ODE
    D = A.diagonal();
    Dinv = numpy.zeros_like(D)
    mask = (D != 0.0)
    Dinv[mask] = 1.0 / D[mask]
    Dinv[D == 0] = 1.0
    Dinv_A  = scale_rows(A, Dinv, copy=True)
    rho_DinvA = approximate_spectral_radius(Dinv_A)
     
    # Calculate (Atilde^k) naively  
    S = (scipy.sparse.eye(dimen,dimen,format="csr") - (1.0/rho_DinvA)*Dinv_A)
    Atilde = scipy.sparse.eye(dimen, dimen, format="csr")
    for i in range(k):
        Atilde = S*Atilde
    
    # Strength Info should be row-based, so transpose Atilde
    Atilde = Atilde.T.tocsr()

    #====================================================================
    #Construct and apply a sparsity mask for Atilde that restricts Atilde^T to the nonzero pattern
    #  of A, with the added constraint that row i of Atilde^T retains only the nonzeros that are 
    #  also in the same PDE as i.

    mask = A.copy()

    #Only consider strength at dofs from your PDE.  Use mask to enforce this by zeroing out
    #   all entries in Atilde that aren't from your PDE.
    if numPDEs > 1:
        row_length = diff(mask.indptr)
        my_pde = mod(range(dimen), numPDEs)
        my_pde = repeat(my_pde, row_length)
        mask.data[ mod(mask.indices, numPDEs) != my_pde ] = 0.0
        del row_length, my_pde
        mask.eliminate_zeros()

    #Apply mask to Atilde, zeros in mask have already been eliminated at start of routine.
    mask.data[:] = 1.0
    Atilde = Atilde.multiply(mask)
    Atilde.eliminate_zeros()
    Atilde.sort_indices()
    del mask

    #====================================================================
    # Calculate strength based on constrained min problem of
    LHS = mat(zeros((NullDim+1, NullDim+1)), dtype=A.dtype)
    RHS = mat(zeros((NullDim+1, 1)), dtype=A.dtype)
    
    ##
    # Choose tolerance for dropping "numerically zero" values later
    t = Atilde.dtype.char
    eps = numpy.finfo(numpy.float).eps
    feps = numpy.finfo(numpy.single).eps
    geps = numpy.finfo(numpy.longfloat).eps
    _array_precision = {'f': 0, 'd': 1, 'g': 2, 'F': 0, 'D': 1, 'G':2}
    tol = {0: feps*1e3, 1: eps*1e6, 2: geps*1e6}[_array_precision[t]]

    for i in range(dimen):
       
        #Get rowptrs and col indices from Atilde
        rowstart = Atilde.indptr[i]
        rowend = Atilde.indptr[i+1]
        length = rowend - rowstart
        colindx = Atilde.indices[rowstart:rowend]
       
        # Local diagonal of A is used for scale invariant min problem
        D_A = mat(eye(length, dtype=A.dtype))
        if proj_type == "D_A":
            for j in range(length):
                D_A[j,j] = D[colindx[j]] 

        #Find row i's position in colindx, matrix must have sorted column indices.
        iInRow = colindx.searchsorted(i)
   
        if length <= NullDim:
            #Do nothing, because the number of nullspace vectors will  
            #be able to perfectly approximate this row of Atilde.
            Atilde.data[rowstart:rowend] = 1.0
        else:
            #Grab out what we want from Atilde and B.  Put into zi, Bi
            zi = mat(Atilde.data[rowstart:rowend]).T
           
            Bi = Bmat[colindx,:]
   
            #Construct constrained min problem
            LHS[0:NullDim, 0:NullDim] = 2.0*Bi.H*D_A*Bi
            LHS[0:NullDim, NullDim] = D_A[iInRow,iInRow]*Bi[iInRow,:].H  
            LHS[NullDim, 0:NullDim] = Bi[iInRow,:]
            RHS[0:NullDim,0] = 2.0*Bi.H*D_A*zi
            RHS[NullDim,0] = zi[iInRow,0]

            #Calc Soln to Min Problem
            x = mat(pinv(LHS))*RHS
            
            # Calculate best constrained approximation to zi with span(Bi), and filter out 
            # "numerically" zero values.  This is important because we look only at the
            # sign of values below when calculating angle.
            zihat = Bi*x[:-1]
            tol_i = max(abs(zihat))*tol
            zihat.real[ abs(zihat.real) < tol_i ] = 0.0
            if numpy.iscomplexobj(zihat):
                zihat.imag[ abs(zihat.imag) < tol_i ] = 0.0

            # if angle in the complex plane between individual entries is 
            #   greater than 90 degrees, then weak.  We can just look at the
            #   dot product to determine if angle is greater than 90 degrees.
            angle = real(ravel(zihat))*real(ravel(zi)) + imag(ravel(zihat))*imag(ravel(zi))
            angle = angle < 0.0
            angle = array(angle, dtype=bool)
            
            #Calculate approximation ratio
            zi = zihat/zi
           
            # If the ratio is small, then weak connection
            zi[abs(zi) <= 1e-4] = 1e100 

            # If angle is greater than 90 degrees, then weak connection
            zi[angle] = 1e100

            #Calculate Relative Approximation Error
            zi = abs(1.0 - zi)
            
            # important to make "perfect" connections explicitly nonzero
            zi[zi < sqrt_near_zero] = 1e-4                 

            #Calculate and apply drop-tol.  Ignore diagonal by making it very large
            zi[iInRow] = 1e5
            drop_tol = min(zi)*epsilon
            zi[zi > drop_tol] = 0.0
            Atilde.data[rowstart:rowend] = ravel(zi)

    #===================================================================

    # Clean up, and return Atilde
    Atilde.eliminate_zeros()
    Atilde.data = array(real(Atilde.data), dtype=float)
    
    # Set diagonal to 1.0, as each point is strongly connected to itself.
    I = scipy.sparse.eye(dimen, dimen, format="csr")
    I.data -= Atilde.diagonal()
    Atilde = Atilde + I

    # If converted BSR to CSR we return amalgamated matrix with the minimum nonzero for each block 
    # making up the nonzeros of Atilde
    if not csrflag:
        Atilde = Atilde.tobsr(blocksize=(numPDEs, numPDEs))
        
        #Atilde = csr_matrix((data, row, col), shape=(*,*))
        Atilde = csr_matrix((array([ Atilde.data[i,:,:][Atilde.data[i,:,:].nonzero()].min() for i in range(Atilde.indices.shape[0]) ]), \
                             Atilde.indices, Atilde.indptr), shape=(Atilde.shape[0]/numPDEs, Atilde.shape[1]/numPDEs) )

    # Standardized strength values require small values be weak and large
    # values be strong.  So, we invert the algebraic distances computed here
    Atilde.data = 1.0/Atilde.data
    
    # Scale Atilde by the largest magnitude entry in each row
    largest_row_entry = numpy.zeros((Atilde.shape[0],), dtype=Atilde.dtype)
    for i in range(Atilde.shape[0]):
        for j in range(Atilde.indptr[i], Atilde.indptr[i+1]):
            val = numpy.abs(Atilde.data[j])
            if val > largest_row_entry[i]:
                largest_row_entry[i] = val
    
    largest_row_entry[ largest_row_entry != 0 ] = 1.0 / largest_row_entry[ largest_row_entry != 0 ] 
    Atilde = Atilde.tocsr()   
    Atilde = scale_rows(Atilde, largest_row_entry, copy=True)

    return Atilde


def reference_distance_strength_of_connection(A, V, theta=2.0, relative_drop=True):
    ''' 
    Reference routine for distance based strength of connection
    '''
    
    # deal with the supernode case
    if isspmatrix_bsr(A):
        dimen = A.shape[0]/A.blocksize[0]
        C = csr_matrix( (numpy.ones((A.data.shape[0],)), A.indices, A.indptr), \
                               shape=(dimen,dimen))
    else:
        A = A.tocsr()
        dimen = A.shape[0]
        C = A.copy()
        C.data = numpy.real(C.data)
    
    if V.shape[1] == 2:
        three_d = False
    elif V.shape[1] == 3:
        three_d = True

    for i in range(dimen):
        rowstart = C.indptr[i]
        rowend = C.indptr[i+1]
        pt_i = V[i,:]
        for j in range(rowstart, rowend):
            if C.indices[j] == i:
                # ignore the diagonal entry by making it large
                C.data[j] = numpy.finfo(numpy.float).max 
            else:
                # distance between entry j and i
                pt_j = V[C.indices[j],:]
                dist = (pt_i[0] - pt_j[0])**2
                dist += (pt_i[1] - pt_j[1])**2
                if three_d:
                    dist += (pt_i[2] - pt_j[2])**2
                C.data[j] = numpy.sqrt(dist)

        # apply drop tolerance
        this_row = C.data[rowstart:rowend]
        if relative_drop:
            tol_i = theta*this_row.min()
            this_row[this_row > tol_i] = 0.0
        else:
            this_row[this_row > theta] = 0.0

        C.data[rowstart:rowend] = this_row 

    C.eliminate_zeros()
    C = C + 2.0*scipy.sparse.eye(C.shape[0], C.shape[1], format='csr')

    # Standardized strength values require small values be weak and large
    # values be strong.  So, we invert the distances.
    C.data = 1.0/C.data
    
    # Scale C by the largest magnitude entry in each row
    largest_row_entry = numpy.zeros((C.shape[0],), dtype=C.dtype)
    for i in range(C.shape[0]):
        for j in range(C.indptr[i], C.indptr[i+1]):
            val = numpy.abs(C.data[j])
            if val > largest_row_entry[i]:
                largest_row_entry[i] = val
    
    largest_row_entry[ largest_row_entry != 0 ] = 1.0 / largest_row_entry[ largest_row_entry != 0 ] 
    C = C.tocsr()   
    C = scale_rows(C, largest_row_entry, copy=True)

    return C






from pyamg.testing import *

from numpy import matrix, array, diag, arange, ones, sqrt, dot, ravel
from scipy import rand
from pyamg.util.linalg import norm
from scipy.sparse import csr_matrix

from pyamg.gallery import poisson
from pyamg.multilevel import *

def precon_norm(v, ml):
    ''' helper function to calculate preconditioner norm of v '''
    v = ravel(v)
    w = ml.aspreconditioner()*v
    return sqrt(dot(v.conjugate(),w))

class TestMultilevel(TestCase):
    def test_coarse_grid_solver(self):
        cases = []

        cases.append( csr_matrix(diag(arange(1,5,dtype=float))) )
        cases.append( poisson( (4,),  format='csr') )
        cases.append( poisson( (4,4), format='csr') )
      
        from pyamg.krylov import cg
        def fn(A, b):
            return cg(A, b)[0]

        # method should be almost exact for small matrices
        for A in cases:
            for solver in ['splu', 'pinv', 'pinv2', 'lu', 'cholesky', 'cg', fn]:
                s = coarse_grid_solver(solver)

                b = arange(A.shape[0], dtype=A.dtype)

                x = s(A,b)
                assert_almost_equal(A*x, b)

                # subsequent calls use cached data
                x = s(A,b)  
                assert_almost_equal(A*x, b)

    def test_aspreconditioner(self):
        from pyamg import smoothed_aggregation_solver
        from scipy.sparse.linalg import cg
        from pyamg.krylov import fgmres

        A = poisson((50,50), format='csr')
        b = rand(A.shape[0])

        ml = smoothed_aggregation_solver(A)

        for cycle in ['V','W','F']:
            M = ml.aspreconditioner(cycle=cycle)
            x,info = cg(A, b, tol=1e-8, maxiter=30, M=M)
            # cg satisfies convergence in the preconditioner norm
            assert(precon_norm(b - A*x,ml) < 1e-8*precon_norm(b,ml))
        
        for cycle in ['AMLI']:
            M = ml.aspreconditioner(cycle=cycle)
            x,info = fgmres(A, b, tol=1e-8, maxiter=30, M=M)
            # fgmres satisfies convergence in the 2-norm
            assert(norm(b - A*x) < 1e-8*norm(b))


    def test_accel(self):
        from pyamg import smoothed_aggregation_solver
        from pyamg.krylov import cg, bicgstab

        A = poisson((50,50), format='csr')
        b = rand(A.shape[0])

        ml = smoothed_aggregation_solver(A)
        
        # cg halts based on the preconditioner norm
        for accel in ['cg',cg]:
            x = ml.solve(b, maxiter=30, tol=1e-8, accel=accel)
            assert(precon_norm(b - A*x,ml) < 1e-8*precon_norm(b,ml))
            residuals = []
            x = ml.solve(b, maxiter=30, tol=1e-8, residuals=residuals, accel=accel)
            assert(precon_norm(b - A*x,ml) < 1e-8*precon_norm(b,ml))
            #print residuals
            assert_almost_equal(precon_norm(b - A*x,ml), residuals[-1])
        
        # cgs and bicgstab use the Euclidean norm
        for accel in ['bicgstab', 'cgs', bicgstab]:
            x = ml.solve(b, maxiter=30, tol=1e-8, accel=accel)
            assert(norm(b - A*x) < 1e-8*norm(b))
            residuals = []
            x = ml.solve(b, maxiter=30, tol=1e-8, residuals=residuals, accel=accel)
            assert(norm(b - A*x) < 1e-8*norm(b))
            #print residuals
            assert_almost_equal(norm(b - A*x), residuals[-1])


    def test_cycle_complexity(self):
        # four levels
        levels = []
        levels.append( multilevel_solver.level() )
        levels[0].A = csr_matrix(ones((10,10)))
        levels[0].P = csr_matrix(ones((10,5)))
        levels.append( multilevel_solver.level() )
        levels[1].A = csr_matrix(ones((5,5)))
        levels[1].P = csr_matrix(ones((5,3)))
        levels.append( multilevel_solver.level() )
        levels[2].A = csr_matrix(ones((3,3)))
        levels[2].P = csr_matrix(ones((3,2)))
        levels.append( multilevel_solver.level() )
        levels[3].A = csr_matrix(ones((2,2)))

        # one level hierarchy
        mg = multilevel_solver(levels[:1])
        assert_equal(mg.cycle_complexity(cycle='V'), 100.0/100.0) #1
        assert_equal(mg.cycle_complexity(cycle='W'), 100.0/100.0) #1
        assert_equal(mg.cycle_complexity(cycle='AMLI'), 100.0/100.0) #1
        assert_equal(mg.cycle_complexity(cycle='F'), 100.0/100.0) #1

        # two level hierarchy
        mg = multilevel_solver(levels[:2])
        assert_equal(mg.cycle_complexity(cycle='V'), 225.0/100.0) #2,1
        assert_equal(mg.cycle_complexity(cycle='W'), 225.0/100.0) #2,1
        assert_equal(mg.cycle_complexity(cycle='AMLI'), 225.0/100.0) #2,1
        assert_equal(mg.cycle_complexity(cycle='F'), 225.0/100.0) #2,1

        # three level hierarchy
        mg = multilevel_solver(levels[:3])
        assert_equal(mg.cycle_complexity(cycle='V'), 259.0/100.0) #2,2,1
        assert_equal(mg.cycle_complexity(cycle='W'), 318.0/100.0) #2,4,2
        assert_equal(mg.cycle_complexity(cycle='AMLI'), 318.0/100.0) #2,4,2
        assert_equal(mg.cycle_complexity(cycle='F'), 318.0/100.0) #2,4,2
        
        # four level hierarchy
        mg = multilevel_solver(levels[:4])
        assert_equal(mg.cycle_complexity(cycle='V'), 272.0/100.0) #2,2,2,1
        assert_equal(mg.cycle_complexity(cycle='W'), 388.0/100.0) #2,4,8,4
        assert_equal(mg.cycle_complexity(cycle='AMLI'), 388.0/100.0) #2,4,8,4
        assert_equal(mg.cycle_complexity(cycle='F'), 366.0/100.0) #2,4,6,3


class TestComplexMultilevel(TestCase):
    def test_coarse_grid_solver(self):
        cases = []

        cases.append( csr_matrix(diag(arange(1,5))) )
        cases.append( poisson( (4,),  format='csr') )
        cases.append( poisson( (4,4), format='csr') )
        
        # Make cases complex
        cases = [ G+1e-5j*G for G in cases ]
        cases = [ 0.5*(G + G.H) for G in cases ]
        
        # method should be almost exact for small matrices
        for A in cases:
            for solver in ['splu','pinv','pinv2','lu','cholesky','cg']:
                s = coarse_grid_solver(solver)

                b = arange(A.shape[0],dtype=A.dtype)

                x = s(A,b)
                assert_almost_equal( A*x, b)

                # subsequent calls use cached data
                x = s(A,b)  
                assert_almost_equal( A*x, b)





from pyamg.testing import *

from pyamg.gallery.mesh import *

class TestRegularTriangleMesh(TestCase):
    def test_1x1(self):
        assert_raises(ValueError, regular_triangle_mesh, 1,1)

    def test_2x2(self):
        Vert,E2V = regular_triangle_mesh(2, 2)

        assert_equal(Vert, [[ 0.,  0.], 
                            [ 1.,  0.], 
                            [ 0.,  1.], 
                            [ 1.,  1.]])

        assert_equal(E2V, [[0, 3, 2],
                           [0, 1, 3]])
    
    def test_3x2(self):
        Vert,E2V = regular_triangle_mesh(3, 2)

        assert_equal(Vert, [[ 0. ,  0. ],                 
                            [ 0.5,  0. ],
                            [ 1. ,  0. ],
                            [ 0. ,  1. ],
                            [ 0.5,  1. ],
                            [ 1. ,  1. ]])

        assert_equal(E2V, [[0, 4, 3], 
                           [1, 5, 4],
                           [0, 1, 4],
                           [1, 2, 5]])




from pyamg.testing import *
from numpy import array, ravel
from scipy import mat
from scipy.sparse import bsr_matrix
from pyamg.util.BSR_utils import *

class TestBSRUtils(TestCase):
    def test_BSR_Get_Row(self):
        indptr  = array([0,2,3,6])
        indices = array([0,2,2,0,1,2])
        data    = array([1,2,3,4,5,6]).repeat(4).reshape(6,2,2)
        B = bsr_matrix( (data,indices,indptr), shape=(6,6) )
        r,i = BSR_Get_Row(B,2)
        assert_equal(r, mat(array([[3],[3]])))
        assert_equal(i, array([4,5]))

    def test_BSR_Row_WriteScalar(self):
        indptr  = array([0,2,3,6])
        indices = array([0,2,2,0,1,2])
        data    = array([1,2,3,4,5,6]).repeat(4).reshape(6,2,2)
        
        indptr2  = array([0,2,3,6])
        indices2 = array([0,2,2,0,1,2])
        data2    = array([[[ 1,  1], [ 1,  1]],
                          [[ 2,  2], [ 2,  2]],
                          [[ 3,  3], [ 3,  3]],
                          [[ 4,  4], [22, 22]],
                          [[ 5,  5], [22, 22]],
                          [[ 6,  6], [22, 22]]])

        B2 = bsr_matrix( (data2,indices2,indptr2), shape=(6,6) )
        B = bsr_matrix( (data,indices,indptr), shape=(6,6) )
        BSR_Row_WriteScalar(B,5,22)
        diff = ravel((B - B2).data)
        assert_equal(diff.shape[0], 0)

    def test_BSR_Row_WriteVect(self):
        indptr  = array([0,2,3,6])
        indices = array([0,2,2,0,1,2])
        data    = array([1,2,3,4,5,6]).repeat(4).reshape(6,2,2)
        
        indptr2  = array([0,2,3,6])
        indices2 = array([0,2,2,0,1,2])
        data2    = array([[[ 1,  1], [ 1,  1]],
                          [[ 2,  2], [ 2,  2]],
                          [[ 3,  3], [ 3,  3]],
                          [[ 4,  4], [11, 22]],
                          [[ 5,  5], [33, 44]],
                          [[ 6,  6], [55, 66]]])

        B2 = bsr_matrix( (data2,indices2,indptr2), shape=(6,6) )
        B = bsr_matrix( (data,indices,indptr), shape=(6,6) )
        BSR_Row_WriteVect(B,5,array([11,22,33,44,55,66]))
        diff = ravel((B - B2).data)
        assert_equal(diff.shape[0], 0)





"Krylov Solvers"

from info import __doc__

from _gmres import *
from _fgmres import *
from _cg import *
from _cr import *
from _cgnr import *
from _cgne import *
from _bicgstab import *
from _steepest_descent import *
from _minimal_residual import *

__all__ = [s for s in dir() if not s.startswith('_')]
from pyamg.testing import Tester
test = Tester().test



from pyamg.testing import *
from pyamg.krylov import *
from numpy import array, zeros, ones, ravel, dot, abs, sqrt, zeros_like
from scipy import mat, random
from scipy.linalg import solve
from pyamg.util.linalg import norm
import pyamg


class TestSimpleIterations(TestCase):
    def setUp(self):
        self.definite_cases=[]
        self.spd_cases=[]

        # 1x1 
        A = mat([[1.2]])
        b = array([3.9]).reshape(-1,1)
        x0 = zeros((1,1))
        self.definite_cases.append({'A':A, 'b':b, 'x0':x0, 'maxiter':1, 'reduction_factor':1e-10})
        self.spd_cases.append({'A': A, 'b':b, 'x0':x0, 'maxiter':1, 'reduction_factor':1e-10})
        
        # 2x2
        A = mat([[ 4.5,    -1.2],
                 [ 3.4,    6.7]])
        b = array([ -3.2, 5.6]).reshape(-1,1)
        x0 = zeros((2,1))
        self.definite_cases.append({'A':A, 'b':b, 'x0':x0, 'maxiter':4, 'reduction_factor':0.1})
        self.spd_cases.append({'A':A.T*A, 'b':b, 'x0':x0, 'maxiter':4, 'reduction_factor':0.1})
        
        # 2x2 Imaginary
        A = mat([[ 4.5,      -1.2j],
                 [ 3.4+1.2j,  6.7]])
        b = array([ -3.2, 5.6]).reshape(-1,1)
        x0 = zeros((2,1))
        self.spd_cases.append({'A':A.T*A, 'b':b, 'x0':x0, 'maxiter':4, 'reduction_factor':0.1})

        # 4x4 
        A = mat([[  1.2,    0.,   0.,     0.],
                 [  0.,     10.,   2.,     6.],
                 [  0.,     0.,   9.3,  -2.31],
                 [ -4.,     0.,   0.,     11.]])
        b = array([1., 3.9, 0., -1.23]).reshape(-1,1)
        x0 = zeros((4,1))
        self.definite_cases.append({'A':A, 'b':b, 'x0':x0, 'maxiter':4, 'reduction_factor':0.3})
        self.spd_cases.append({'A':A.T*A, 'b':b, 'x0':x0, 'maxiter':4, 'reduction_factor':0.3})
        
        # 4x4 Imaginary
        A = mat(A, dtype=complex)
        A[0,0] += 3.1j
        A[3,3] -= 1.34j
        A[1,3] *= 1.0j
        A[1,2] += 1.0j
        b = array([1. - 1.0j, 2.0 - 3.9j, 0., -1.23]).reshape(-1,1)
        x0 = ones((4,1))
        self.spd_cases.append({'A':A.H*A, 'b':b, 'x0': x0, 'maxiter':4, 'reduction_factor':0.3})

        # 10x10 
        A = mat([[91.1,    0.,   0.,   0.,  3.9,  0.,   0.,  11.,  -1.,  0.],
                 [  0.,   45.,   2.9,  0.,   0.,  6.8,  0.,  0.,    0.,  0.],
                 [  0.,    0.,  19.0,  0.,   0.,  0.8,  1., -2.2,   0.,  9.],
                 [ -4.,    0.,   0.0,  4.,   0.,  0.0,  2.,  2.2,   0.,  0.],
                 [  0.,    0.,   0.0, 21.,  30.,  0.1,  0.,   0.,   0.,  0.],
                 [  0.,    0.,   0.0,  0.,  -4.7, 7.0,  0.,   0.,   0.,  0.],
                 [  2.1,   7.,   2.0,  0.,   0.,  0.0,  8.,   0.,   0.,  0.],
                 [  0.,    0.,   0.0, 34.,   0.,  0.0,  0., 87.1,  -12.3,0.],
                 [  0.,   3.4,   0.0,  0.,   0., -0.3,  0.,   0.,   7.,  0.],
                 [  9.,    0.,   0.0,  0.,  8.7,  0.0,  0.,   0.,   0., 11.2]])
        b = array([1., 0., 0.2, 8., 0., -1.9, 11.3, 0.0, 0.1, 0.0]).reshape(-1,1)
        x0 = zeros((10,1))
        x0[4] = 11.1; x0[7] = -2.1
        self.definite_cases.append({'A':A, 'b':b, 'x0':x0, 'maxiter':2, 'reduction_factor':0.98})
        self.spd_cases.append({'A' : mat(pyamg.gallery.poisson((10,)).todense()), 'b':b, 'x0':x0, 'maxiter':2, 'reduction_factor':0.5})

    def test_steepest_descent(self):
        # Ensure repeatability
        random.seed(0)
        

        for case in self.spd_cases:
            A=case['A']; b=case['b']; x0=case['x0']; maxiter=case['maxiter']
            reduction_factor=case['reduction_factor']

            # This function should always decrease 
            fvals = []
            def callback(x):
                fvals.append( 0.5*dot(ravel(x), ravel(A*x.reshape(-1,1)) ) - dot(ravel(b), ravel(x)) )

            (x,flag) = steepest_descent(A,b, x0=x0,tol=1e-16,maxiter=maxiter,callback=callback)
            actual_factor = ( norm(ravel(b) - ravel( A*x.reshape(-1,1))) /    \
                              norm(ravel(b) - ravel(A*x0.reshape(-1,1))) )
            assert( actual_factor < reduction_factor)
            
            if A.dtype != complex:
                for i in range(len(fvals)-1):
                    assert(fvals[i+1] <= fvals[i])

        
        # Test preconditioning
        A = pyamg.gallery.poisson((10,10),format='csr') 
        b = rand(A.shape[0],1) 
        x0 = rand(A.shape[0],1) 
        fvals = []
        def callback(x):
            fvals.append( 0.5*dot(ravel(x), ravel(A*x.reshape(-1,1)) ) - dot(ravel(b), ravel(x)) )
        #
        resvec=[]
        sa = pyamg.smoothed_aggregation_solver(A)
        (x,flag) = steepest_descent(A,b,x0,tol=1e-8,maxiter=20,residuals=resvec,
                                    M=sa.aspreconditioner(), callback=callback)
        assert( resvec[-1] < 1e-8)
        for i in range(len(fvals)-1):
            assert(fvals[i+1] <= fvals[i])
    
    def test_minimal_residual(self):
        # Ensure repeatability
        random.seed(0)
        
        self.definite_cases.extend(self.spd_cases)

        for case in self.definite_cases:
            A=case['A']; maxiter=case['maxiter']
            x0=rand(A.shape[0],)
            b=zeros_like(x0)
            reduction_factor=case['reduction_factor']
            if A.dtype != complex:

                # This function should always decrease (assuming zero RHS)
                fvals = []
                def callback(x):
                    fvals.append( sqrt(dot( ravel(x), ravel(A*x.reshape(-1,1)) )) )
                #
                (x,flag) = minimal_residual(A,b, x0=x0,tol=1e-16,maxiter=maxiter,callback=callback)
                actual_factor = ( norm(ravel(b) - ravel( A*x.reshape(-1,1))) /    \
                                  norm(ravel(b) - ravel(A*x0.reshape(-1,1))) )
                assert( actual_factor < reduction_factor )
                if A.dtype != complex:
                    for i in range(len(fvals)-1):
                        assert(fvals[i+1] <= fvals[i])

        
        # Test preconditioning
        A = pyamg.gallery.poisson((10,10),format='csr') 
        x0 = rand(A.shape[0],1) 
        b = zeros_like(x0) 
        fvals = []
        def callback(x):
            fvals.append( sqrt(dot( ravel(x), ravel(A*x.reshape(-1,1)) )) )
        #
        resvec=[]
        sa = pyamg.smoothed_aggregation_solver(A)
        (x,flag) = minimal_residual(A,b,x0,tol=1e-8,maxiter=20,residuals=resvec,
                                    M=sa.aspreconditioner(), callback=callback)
        assert( resvec[-1] < 1e-8)
        for i in range(len(fvals)-1):
            assert(fvals[i+1] <= fvals[i])





"""PyAMG: Algebraic Multigrid Solvers in Python"""

from info import __doc__

__doc__ += """

Utility tools
-------------

  test        --- Run pyamg unittests (requires nose unittest framework)
  bench       --- Run pyamg benchmarks (requires nose unittest framework)
  __version__ --- pyamg version string

"""

# We first need to detect if we're being called as part of the numpy setup
# procedure itself in a reliable manner.
try:
    __PYAMG_SETUP__
except NameError:
    __PYAMG_SETUP__ = False


if __PYAMG_SETUP__:
    import sys as _sys
    _sys.stderr.write('Running from numpy source directory.\n')
    del _sys
else:
    try:
        from __config__ import show as show_config
    except ImportError, e:
        msg = """Error importing pyamg: you cannot import pyamg while
        being in pyamg source directory; please exit the pyamg source
        tree first, and relaunch your python interpreter."""
        raise ImportError(msg)

    # Emit a warning if numpy is too old
    import numpy as _numpy
    majver, minver = [float(i) for i in _numpy.version.version.split('.')[:2]]
    if majver < 1 or (majver == 1 and minver < 2):
        import warnings
        warnings.warn("Numpy 1.2.0 or above is recommended for this version of\
                      PyAMG (detected version %s)" % _numpy.version.version,
                      UserWarning)

    # Emit a warning if scipy is too old
    import scipy as _scipy
    majver, minver = [float(i) for i in _scipy.version.version.split('.')[:2]]
    if minver < 0.7:
        import warnings
        warnings.warn("SciPy 0.7 or above is recommended for this version of\
                      PyAMG (detected version %s)" % _scipy.version.version,
                      UserWarning)

    del _numpy, _scipy

    from version import git_revision as __git_revision__
    from version import version as __version__

    from multilevel import *
    from classical import ruge_stuben_solver
    from aggregation import smoothed_aggregation_solver, rootnode_solver
    from gallery import demo
    from blackbox import solve, solver, solver_configuration

    __all__ = [s for s in dir() if not s.startswith('_')]
    __all__ += ['test', '__version__']
    from pyamg.testing import Tester
    test = Tester().test
    bench = Tester().bench
